@@ -28,7 +28,7 @@ use constant SAFEWRAP => ( Safe->can("wrap_code_ref") ? 1 : 0 );
 #inherits Apache::Session
 #link Lemonldap::NG::Common::Apache::Session::SOAP protected globalStorage
 
-our $VERSION = '1.0.2';
+our $VERSION = '1.1.0';
 
 our %EXPORT_TAGS;
 
@@ -48,7 +48,7 @@ our (
     $customFunctions,    $transform,           $cda,
     $childInitDone,      $httpOnly,            $cookieExpiration,
     $timeoutActivity,    $datasUpdate,         $useRedirectOnForbidden,
-    $useRedirectOnError,
+    $useRedirectOnError, $useSafeJail,
 );
 
 ##########################################
@@ -65,6 +65,7 @@ BEGIN {
               $locationCondition $defaultCondition $locationCount
               $locationProtection $defaultProtection $datasUpdate
               $locationRegexp $apacheRequest $datas safe $customFunctions
+              $useSafeJail
               )
         ],
         import  => [qw( import @EXPORT_OK @EXPORT %EXPORT_TAGS )],
@@ -79,7 +80,7 @@ BEGIN {
             qw( MP OK REDIRECT FORBIDDEN DONE DECLINED SERVER_ERROR
               $useRedirectOnForbidden $useRedirectOnError )
         ],
-        post    => [qw($transform)],
+        post    => [qw($transform postFilter)],
         cda     => ['$cda'],
         cookie  => [qw($cookieName $https $httpOnly $cookieExpiration)],
         session => ['$timeoutActivity'],
@@ -146,6 +147,7 @@ BEGIN {
             threads::shared::share($timeoutActivity);
             threads::shared::share($useRedirectOnForbidden);
             threads::shared::share($useRedirectOnError);
+            threads::shared::share($useSafeJail);
         };
     }
     elsif ( MP() == 1 ) {
@@ -327,7 +329,8 @@ sub safe {
 
     return $safe if ($safe);
 
-    $safe = new Safe;
+    $useSafeJail = 1 unless defined $useSafeJail;
+
     my @t = $customFunctions ? split( /\s+/, $customFunctions ) : ();
     foreach (@t) {
         $class->lmLog( "Custom function : $_", 'debug' );
@@ -346,14 +349,36 @@ sub safe {
             }";
         $class->lmLog( $@, 'error' ) if ($@);
     }
-    $safe->share_from( 'main', ['%ENV'] );
-    $safe->share_from( 'Lemonldap::NG::Common::Safelib',
-        $Lemonldap::NG::Common::Safelib::functions );
-    $safe->share( '&encode_base64', '$datas', '&portal', '$apacheRequest', @t );
+
+    if ($useSafeJail) {
+        $safe = new Safe;
+        $safe->share_from( 'main', ['%ENV'] );
+        $safe->share_from( 'Lemonldap::NG::Common::Safelib',
+            $Lemonldap::NG::Common::Safelib::functions );
+        $safe->share( '&encode_base64', '$datas', '&portal', '$apacheRequest',
+            @t );
+    }
+    else {
+        $safe = $class;
+    }
 
     return $safe;
 }
 
+## @method reval
+# Fake reval method if useSafeJail desactivated
+sub reval {
+    my ( $class, $e ) = splice @_;
+    return eval $e;
+}
+
+## @method wrap_code_ref
+# Fake wrap_code_ref method if useSafeJail desactivated
+sub wrap_code_ref {
+    my ( $class, $e ) = splice @_;
+    return $e;
+}
+
 ## @imethod void localInit(hashRef args)
 # Call purgeCache() to purge the local cache, launch the status process
 # (statusProcess()) in wanted and launch childInit().
@@ -580,6 +605,10 @@ sub defaultValuesInit {
       defined($useRedirectOnForbidden)
       ? $useRedirectOnForbidden
       : $args->{useRedirectOnForbidden};
+    $useSafeJail =
+      defined($useSafeJail)
+      ? $useSafeJail
+      : $args->{useSafeJail};
     1;
 }
 
@@ -1331,40 +1360,47 @@ sub postUrlInit {
             $tmp .= "'$_'=>$postdata{$_},";
         }
 
-        # Build subroutine
-        my $sub = "sub{
-            my \$f = shift;
-            my \$l;
-            unless(\$f->ctx){
-            \$f->ctx(1);
-            my \$u=URI->new('http:');
-            \$u->query_form({$tmp});
-            my \$s=\$u->query();
-            \$l = \$f->r->headers_in->{'Content-Length'};
-            \$f->r->headers_in->set( 'Content-Length' => length(\$s) );
-            \$f->r->headers_in->set( 'Content-Type' => 'application/x-www-form-urlencoded' );
-            \$f->print(\$s);
-            while ( \$f->read( my \$b, \$l ) ) {}
-            \$f->seen_eos(1);
-            }
-            return OK;
-        }"
-          ;
-        $sub = (
-            SAFEWRAP
-            ? $class->safe->wrap_code_ref( $class->safe->reval($sub) )
-            : $class->safe->reval($sub)
-        );
         $class->lmLog( "Compiling POST request for $url", 'debug' );
         $transform->{$url} = sub {
             return $class->buildPostForm($url)
               if ( $apacheRequest->method ne 'POST' );
-            $apacheRequest->add_input_filter($sub);
+            $apacheRequest->add_input_filter(
+                sub {
+                    $class->postFilter( $tmp, @_ );
+                }
+            );
             OK;
           }
     }
 }
 
+## @rmethod protected int postFilter(hashref data, Apache2::Filter f)
+# POST data
+# @param $data Data to POST
+# @param $f Current Apache2::Filter object
+# @return Apache2::Const::OK
+sub postFilter {
+    my $class = shift;
+    my $data  = shift;
+    my $f     = shift;
+    my $l;
+
+    unless ( $f->ctx ) {
+        $f->ctx(1);
+        my $u = URI->new('http:');
+        $u->query_form( { $class->safe->reval($data) } );
+        my $s = $u->query();
+        $l = $f->r->headers_in->{'Content-Length'};
+        $f->r->headers_in->set( 'Content-Length' => length($s) );
+        $f->r->headers_in->set(
+            'Content-Type' => 'application/x-www-form-urlencoded' );
+        $f->print($s);
+        while ( $f->read( my $b, $l ) ) { }
+        $f->seen_eos(1);
+    }
+    return OK;
+}
+
 ## @rmethod protected transformUri(string uri)
 # Transform URI to replay POST forms
 # @param uri URI to catch
